#!/usr/bin/perl
#
# check_ifoperstatus2.pl - netsaint plugin 
# 
#
#
#
# Copyright (C) 2000 Christoph Kron
# Changed to support interface descriptions instead of keys 
# by Vladimir Ivaschenko <vi@maks.net> 2001
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Report bugs to: ck@zet.net
#
# 11.01.2000 Version 1.0

#use strict;

use Getopt::Long;

use SNMP; $SNMP::auto_init_mib = 0;

use SNMP::Util;
use Data::Dumper;

$ENV{'MIBDIRS'} = '/usr/share/snmp/mibs';


&Getopt::Long::config('auto_abbrev');



my $status;
my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN' , '-1',
              'OK' , '0',
              'WARNING', '1',
              'CRITICAL', '2');

my %ifOperStatus = 	('1','up',
			 '2','down',
			 '3','testing',
			 '4','unknown',
			 '5','dormant',
			 '6','notPresent');

my $state = "UNKNOWN";
my $answer = "";
my $snmpkey = 1;
my $community = "public";
my $port = 161;
my @snmpoids;
my $snmpIfOperStatus;
my $snmpLocIfDescr;
my $hostname;
my $snmp;
my $error;
my $response;
my $responsen;


sub usage {
  printf "\nMissing arguments!\n";
  printf "\n";
  printf "Perl Check IfOperStatus plugin for NetSaint\n";
  printf "Checks operational status of specified interface\n\n";
  printf "USAGE: \n";
  printf "check_ifoperstatus2.pl [-k <IF_KEY>] [-d <IF_DESCR>] -c <READCOMMUNITY> <HOSTNAME>";
  printf "\nCopyright (C) 2000 Christoph Kron\n";
  printf "\nCopyright (C) 2001 Vladimir Ivaschenko\n";
  printf "check_ifoperstatus2.pl comes with ABSOLUTELY NO WARRANTY\n";
  printf "This programm is licensed under the terms of the ";
  printf "GNU General Public License\n(check source code for details)\n";
  printf "\n\n";
  exit $ERRORS{"UNKNOWN"};
}

sub fetch_ifdescr {

    @result = $snmp->walk( -format => 'ne',
                       -oids   => ['ifDescr'],
                       -print  => 'off');

    for($i = 0; $i < @result; $i=$i+2) {
           $result[$i] = substr($result[$i],8, length($result[$i]) - 7);
           $interfaces{$result[$i+1]} = $result[$i];
    }
}


# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     print ("ERROR: No snmp response from $hostname (alarm)\n");
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);


$status = GetOptions("key=i",\$snmpkey,
		     "descr=s",\$ifdescr,
                     "community=s",\$community);
if ($status == 0)
{
        &usage;
}
  
   #shift;
   $hostname  = shift || &usage;

$snmp = new SNMP::Util(        -device => $hostname,
                       -community => $community,
                       -timeout => 15,
                       -retry => 2,
                       -poll => 'on',
                       -poll_timeout => 5,
                       -verbose => 'off',
                       -errmode => 'return',
                       -delimiter => ' ');

#   ($session, $error) = Net::SNMP->session(
#      -hostname  => $hostname,
#      -community => $community,
#      -port      => $port
#   );

   if (!defined($snmp)) {
      $state='UNKNOWN';
      $answer=$error;
      print ("$state: $answer");
      exit $ERRORS{$state};
   }

   if (defined $ifdescr) {
       fetch_ifdescr($hostname, $community);
       $snmpkey=$interfaces{$ifdescr};
   }
       
#   $snmpLocIfDescr = '1.3.6.1.4.1.9.2.2.1.1.28' . "." . $snmpkey;
   $snmpIfOperStatus = 'ifOperStatus.'.$snmpkey;
   $snmpIfDescr = 'ifDescr.'.$snmpkey;


   push(@snmpoids,$snmpIfOperStatus);

   if (!defined(@response = $snmp->get('v', @snmpoids))) {
      $answer=$snmp->errmsg;
      $state = 'CRITICAL';
      print ("$state: $answer,$community,$snmpkey");
      exit $ERRORS{$state};
   }

   push(@snmpoids,$snmpIfDescr);

   if (defined(@responsen = $snmp->get('v', @snmpoids))) {
      @response=@responsen;
   }
   else
       {
       undef @response;
       }

   $answer = sprintf("host '%s',%s(%s) is %s\n", 
      $hostname, 
      $response[1],
      $snmpkey, 
      $ifOperStatus{$response[0]}
   );

   if ( $response[0] == 1 ) {
      $state = 'OK';
   }
   else {
	$state = 'CRITICAL';
   }

print ("$state: $answer");
exit $ERRORS{$state};

